from airflow_provider_census.operators.census import CensusOperator
from airflow_provider_census.sensors.census import CensusSensor

from airflow import DAG
from airflow.utils.dates import days_ago
from datetime import timedelta

default_args = {
    'owner': 'airflow',
    'depends_on_past': False,
    'start_date': days_ago(1),
    'email': ['airflow@example.com'],
    'email_on_failure': False,
    'email_on_retry': False,
    'retries': 1,
    'retry_delay': timedelta(minutes=5),
}

dag = DAG('census', max_active_runs=1, default_args=default_args)

sync = CensusOperator(
    task_id='sync',
    census_conn_id='census_default',
    sync_id=4895,
    dag=dag,
)

sensor = CensusSensor(
    task_id='sensor',
    sync_run_id="{{ ti.xcom_pull(task_ids = 'sync') }}",
    census_conn_id='census_default',
    dag=dag,
)

sync >> sensor



def get_provider_info():
    return {
        'package-name': 'airflow-provider-census',
        'name': 'Census Provider',
        'description': 'A Census provider for Apache Ariflow.',
        'hook-class-names': ['airflow_provider_census.hooks.census.CensusHook'],
        'versions': ['1.2.0']
    }



try:
    from airflow.sensors.base import BaseSensorOperator # airflow 2.0
except ImportError:
    from airflow.sensors.base_sensor_operator import BaseSensorOperator # airflow 1.10

from airflow.exceptions import AirflowException
from airflow_provider_census.hooks.census import CensusHook
from airflow.utils.decorators import apply_defaults

from airflow_provider_census.hooks.census import CensusHook


class CensusSensor(BaseSensorOperator):
    """Waits for sync to complete.


    :param sync_run_id: Census sync run ID
    :type sync_run_id: str
    :param census_conn_id: `Conn ID` of the Connection to be used to configure this hook.
    :type census_conn_id: str
    """

    template_fields = ['sync_run_id']

    @apply_defaults
    def __init__(self, sync_run_id, census_conn_id='census_default', **kwargs):
        super().__init__(**kwargs)
        self.sync_run_id = sync_run_id
        self.census_conn_id = census_conn_id
        self.hook = None

    def _get_hook(self) -> CensusHook:
        if self.hook is None:
            self.hook = CensusHook(census_conn_id=self.census_conn_id)
        return self.hook

    def poke(self, context):
        hook = self._get_hook()
        info = hook.get_sync_run_info(self.sync_run_id)
        status = info['status']

        if status == 'failed':
            raise AirflowException(
                'Census sync run {sync_run_id} failed with error: {error_message}'.format(
                    sync_run_id=self.sync_run_id,
                    error_message=info['error_message']
                )
            )
        else:
            return status == 'completed'



from airflow.models import BaseOperator
from airflow.utils.decorators import apply_defaults
from typing import Any

from airflow_provider_census.hooks.census import CensusHook


class CensusOperator(BaseOperator):
    """Triggers sync with Census API.

    :param sync_id: Census sync ID
    :type sync_id: int
    :param census_conn_id: `Conn ID` of the Connection to be used to configure this hook.
    :type census_conn_id: str
    """

    @apply_defaults
    def __init__(self, sync_id: int, census_conn_id: str = 'census_default', **kwargs):
        super().__init__(**kwargs)
        self.sync_id = sync_id
        self.census_conn_id = census_conn_id

    def _get_hook(self) -> CensusHook:
        return CensusHook(census_conn_id=self.census_conn_id)

    def execute(self, context) -> Any:
        hook = self._get_hook()

        return hook.trigger_sync(self.sync_id)



import requests
from typing import Dict

from airflow.exceptions import AirflowException
try:
    from airflow.providers.http.hooks.http import HttpHook # airflow 2.0
except ImportError:
    from airflow.hooks.http_hook import HttpHook # airflow 1.10

class CensusHook(HttpHook):
    """Census API hook

    :param census_conn_id: `Conn ID` of the Connection to be used to configure this hook.
    :type census_conn_id: str
    """

    conn_name_attr = 'census_conn_id'
    default_conn_name = 'census_default'
    conn_type = 'census'
    hook_name = 'Census'

    @staticmethod
    def get_ui_field_behaviour() -> Dict:
        return {
            'hidden_fields': ['login', 'port', 'schema', 'extra', 'host'],
            'relabeling': {
                'password': 'Census Secret Token'
            }
        }

    def __init__(self, census_conn_id: str = default_conn_name, *args, **kwargs) -> None:
        super().__init__(http_conn_id = census_conn_id, *args, **kwargs)
        self.census_conn_id = census_conn_id

    def _get_secret_token(self) -> str:
        conn = self.get_connection(self.http_conn_id)
        secret_token = conn.password
        if not secret_token:
            raise AirflowException('Census Secret Token is required for this hook.')
        return secret_token

    def get_conn(self, headers) -> requests.Session:
        secret_token = self._get_secret_token()

        auth_header = {
            'Authorization': 'Bearer {secret_token}'.format(secret_token = secret_token)
        }

        all_headers = {**auth_header, **headers} if headers else auth_header

        session = super().get_conn(all_headers)

        conn = self.get_connection(self.census_conn_id)
        if not conn.host:
            self.base_url = 'https://app.getcensus.com'

        return session

    def trigger_sync(self, sync_id):
        endpoint = 'api/v1/syncs/{sync_id}/trigger'.format(sync_id = sync_id)

        self.method = 'POST'
        response = self.run(endpoint)
        response.raise_for_status()

        return response.json()['data']['sync_run_id']

    def get_sync_run_info(self, sync_run_id):
        endpoint = 'api/v1/sync_runs/{sync_run_id}'.format(sync_run_id = sync_run_id)

        self.method = 'GET'
        response = self.run(endpoint)
        response.raise_for_status()

        return response.json()['data']
